// Definimos las variables del programa 
float T; // Periode de la oscilación
float t, t0; // Tiempo inicial (t0) y variable tiempo
float A; // Amplitud de la oscilación
float omega; // Velocidad angular de la vibración
float lambda;  // Longitud de onda
float v; // Velocidad de propagación de la onda

boolean clic; // La variable clic nos dice si hemos clicado a la pantalla
PVector r0; // Posición donde clicamos a la pantalla

void setup() {
  // Creamos la pantalla donde visualizaremos la onda
  size(600, 600);
  // Inicializamos variables
  T = 0.5; // Definimos el periodo en segundos
  omega = 2*PI/T; // Calculamos omega en función del periodo
  lambda = 50; // Definimos la longitud de onda en píxeles
  v = lambda / T; // Calculamos la velocidad de propagación a partir del periode, T, y la longitud de onda, lambda.
  
  A = 128; // Amplitud de la onda. En este caso es la mitad del rango de tonalidades de gris (256)
  
  t = t0 = 0.;  // Inicializamos el tiempo inicial y la variable tiempo a cero
  
  clic = false; // Al empezar el programa todavía no hemos clicado a la pantalla
  
  r0 = new PVector(0,0); // Puesto que odavía no hemos clicado, definimos la posición inicialn de la onda en el punto (0,0) (en píxeles), 
                         // aunque lo podríamos hacer en cualquier otro punto.
}

void draw() {
  background(0); // Borramos el fotograma anterior
  if(clic) // Solo entramos en esta parte del código si hemos clicado sobre la pantalla.  
           // Si no lo hemos hecho, no haremos nada.
  {
    float dist;
    int gris;
    t = (millis()/1000. - t0); // Calculamos el tiempo que ha pasado desde que hemos clicado
    for(int i = 0; i < width; i++){
      for (int j = 0; j < height; j++){ // Hacemos un loop para pasar por todos los píxeles de la pantalla
        dist = sqrt((r0.x-i)*(r0.x-i)+(r0.y-j)*(r0.y-j)); // Calculamos la distancia desde el punto que hemos clicado a cada píxel
        if(dist>v*t){
          gris = 0; // Si la distancia es más grande que la distancia que ha recorrido la onda hasta el momento, 
                    // no cambiemos el color del píxel
        }else{ // Si la distancia del píxel en el centro de la onda es más pequeña que la distancia recorrida por la onda, 
               // entonces utilizamos la ecuación de la onda para cambiar el color del píxel.
          gris = (A/2)*(1+sin((2*PI*dist/lambda)-(omega*t)-PI/2)); // Fijaos que el color del píxel no puede tener un valor negativo, 
                                                                   // por lo tanto, tenemos que modificar la función sinus. 
                                                                   // El valor más negativo que puede tener es -1, por lo tanto le sumamos 1.
                                                                   // El valor más positivo que puede tener es 1, si sumamos 1 tendremos 2. 
                                                                   // Por lo tanto dividimos la amplitud por 2.
        }
        set(i, j, color(gris, gris, gris)); // Una vez hemos calculado el valor del color del píxel, lo cambiamos.
      }
    }
  }
}

void mouseClicked(){ // Cuando hacemos clic
  clic = !clic; // Cambiamos el valor de la variable clic
  if(clic) // Si hemos clicado
  {
    t0 = millis()/1000.; // Calculamos el valor del tiempo en este momento, y le asignamos a la variable t0.
    // También guardamos la posición del ratón en este instante
    r0.x = mouseX; 
    r0.y = mouseY;
  }
}
