// En este programa mostraremos un conjunto de partículas o bolas que están dispuestas en linia.
// Cada una de estas partículas vibra siguiendo un MOHS.
// Todas juntas se mueven como lo hace una ola que se propaga, por ejemplo, en la superficie de 
// el agua de un acuario, como se puede ver en sus paredes si son transparentes.

ArrayList<CVibrPart> onda = new ArrayList<CVibrPart>(); 
  // Generamos una colección (array) de partículas. 
  // Una colección (array) se un número de elementos que (podrían ser variables int, float, o el que sea) y que 
  // se guardan dentro de una variable, que en este caso hemos denominado 'onda'. 
  // En nuestro caso, el tipo de variable que guardamos dentro de la colección 'onda' es un conjunto de variables 
  // del tipo CVibrPart, que hemos creado nosotros.
  // Las colecciones son una manera muy conveniente de gestionar multiples variables, sobre todo
  // en el supuesto de que estas variables no sean propias de Processing (como por ejemplo int, float, etc)
  // si no que son variables más complejas. Este es nuestro caso.

int iNum = 30; // Número de partículas

void setup()
{
  size(630, 300); // Creamos la pantalla

  // Generamos 30 bolas (antes hemos hecho iNum = 30). 
  // Por no tener que repetir el mismo código 30 veces, hacemos iteraciones.
  for(int i =0; i<iNum; i++){
    // Definimos las variables que necesitamos para generar una nueva partícula
    int x = 20*(i+1); // la posición x inicial (las distribuimos por el ancho de la pantalla)
    int y = 150; // la posición y inicial (las ponemos todas a la misma altura)
    float A = 30; // La amplitud de la vibración será de 30 píxeles
    float phase = i*20*PI/180; // Definimos la fase entre las diferentes bolas. 
                               // Esta fase no es aleatoria, sino que depende de la posición de cada bola
    float f = 1; // Definimos la frecuencia de la vibración (1Hz, o una oscilación por segundo)
      // Ahora creamos una nueva bola con estos parámetros.
    
    CVibrPart part; // Definimos una partícula. 
      // Este es el primer paso para utilizar el código de la clase CVibrPart que hemos creado. En programación, 
      // la variable 'part' es una 'instancia de la clase CVibrPart', o 'un objeto de la clase CVibrPart'.
      // Si queréis saber algo más sobre qué es una clase, podéis mirar el fichero 'CVibrPart'.

      // El segundo paso para crear un objeto de la clase CVibrPart es la linia siguiente
      // Si os acordais, los dos pasos para generar vector del tipo PVector (que ya hemos trabajado antes)
      // eran muy parecidos.
    part = new CVibrPart(x, y, A, f, phase);
    
      // Una vez hemos generado el objeto de la clase 'CVibrPart', lo queremos añadir a la colección que hemos creado antes
      // Para eso, utilizaremos la función 'get' de la clase 'ArrayList'.
      // Parece complicado, pero ahora veréis que no lo es tanto. 
      // Antes hemos creado una colección de objetos 'CVibrArray', y lo hemos denominado 'onda'.
      // Para añadir un objeto a la colección, llamaremos la función 'add' escribiendo 'ola.add', y como variable
      // pondremos el objeto 'CVibrPart' que justo hemos creado, y que hemos denominado 'part'.
    onda.add(part); // Acabamos de añadir el objeto que hemos guardado a 'part', dentro de la colección 'onda'.
      // Fijaos en un detalle: no hemos creado un objeto 'CVibrPart' nuevo (no hemos utilizado el comando 'new'). 
      // Esto es porque la partícula ya estaba creada, y guardada dentro de la colección onda.
      // Este es un detalle importante!
  }
}

void draw(){
  background(0); // Borramos la pantalla
  float time = float(millis())/1000.; // Calculamos el instante de tiempo en el que nos encontramos, y lo ponemos en segundos
  PVector pvPos; // Creamos un vector que llenaremos con la posición que nos dé la clase CVibrPart
  for(int i =0; i<iNum; i++){ // Repetimos el siguiente código para cada partícula en la colección (array) 'onda'.
      // Lo que haremos aquí será calcular la posición de cada partícula con la función 'pvOnaLongitudinal' de la clase 
      // 'CVibrPart'. Antes hemos llamado la función 'get' relacionada al objeto 'onda' escribiendo 'onda.get'
      // Ahora lo haremos de una manera parecida. Primero creamos una variable del tipo CVibrPart
    
    CVibrPart part;
    
    // Ahora cogeremos la partícula y de la colección 'onda', utilizando la función 'get'
    // Lo hacemos igual que antes, con un punto:
    part = onda.get(i); // Recuperamos la partícula que se encuentra a la posición 'y' de la colección 'onda'. 
    
    pvPos = part.pvOnaTransversal(time); // Llamamos a la función 'pvOnaTransversal' del objeto 'part' que 
      // acabamos de extraer de 'onda'.
      // Esta función devuelve la posición en cada momento de una partícula que se mueve según un MOHS con las 
      // características con las que se creó
    
    ellipse(pvPos.x, pvPos.y, 5, 5); // Una vez calculada la posición, dibujamos un círculo en aquella posición.
  }  
}
