///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Clases
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Esta es la definición de una clase que denominamos CVibrPart.
// Una clase es el código que define un tipo de variable nueva (pareciendo al caso de los vectores PVector).
// Este tipo de 'variables nuevas' pueden tener características asociadas, que se denominan 'variables miembro'.
// Las clases también pueden definir funciones que se ejecutan relacionadas con estas variables miembro.

// En este fichero se define una clase que describe el comportamiento de cada una de las bolas de la animación.

// Las clases son la base de la Programación Orientada a Objetos (POO, o Object Oriented Programming, o OOP en inglés).
// En POO lo que hacemos es describir los componentes del programa y sus funcionalidades en clases. Así el programa en sí
// queda más simple, y el código de la clase lo podemos utilizar en otros programas de manera más fácil.

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Clases Vs Objetos
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Las clases son código donde se define el comportamiento.
// Cuando en un programa se genera una variable de esta clase, o como se llama en programación 'se instancia una clase',
// se crea un objeto. De alguna manera, la clase es abstracta, y el objeto es concreto.

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class CVibrPart{
  // En esta parte del código definimos las variables propias, que definen cada una de las bolas de la animación
  PVector m_pvIniPos; // Posición del centro de la oscilación de la partícula
  
  float m_fAmpl; // Amplitud de la oscilación
  float m_fFreq; // Frecuencia de la oscilación
  
  float m_fPhase; // Desfase de la oscilación
  
  // Constructor
  /////////////////////////////////////////////////////////////////////
  // El constructor de una clase es la función que se llama
  // automáticamente cuando se crea una variable de este tipo (o cuando 
  // se 'instancia la clase').
  // En nuestro caso, el constructor se llama cuando se crea una variable 
  // del tipo CVibrPart).
  // Para generar una de estas bolas necesitamos:
  //    - La posición central de la vibración
  //    - La amplitud de la vibración
  //    - La fase
  //    - La frecuencia de la vibración
  /////////////////////////////////////////////////////////////////////
  CVibrPart(float x, float y, float fAmpl, float fFreq, float fPhase){
    m_pvIniPos = new PVector(0,0); // Generamos el vector que da la posición del centro de la vibración de la partícula.
    // Guardamos la posición x e y del centro de la vibración en el vector que hemos creado.
    m_pvIniPos.x = x; 
    m_pvIniPos.y = y;
    
    m_fAmpl = fAmpl; // Guardamos la amplitud
    m_fPhase = fPhase; // La fase
    m_fFreq = fFreq; // Y la frecuencia.
      // Fijaos en un detalle: la variable 'fAmpl' la tenemos que proporcionar al generar el objeto. El valor que proporcionamos
      // se guarda en la variable miembro 'm_fAmpl', y queda allá guardada. 
      // El prefijo 'm_' quiere decir que es variable miembro de la clase.
  }
  
  
  // pvOnaTransversal
  /////////////////////////////////////////////////////////////////////
  // Esta función devuelve el valor de la posición de la partícula que vibra según los parámetros que se le han asignado.
  // Este cálculo se hace para el instante de tiempo concreto en el que se calcula.
  // La onda en este caso se propaga en la dirección 
  /////////////////////////////////////////////////////////////////////
  PVector pvOnaTransversal(float fT){
    
    PVector pvPos = new PVector(0,0); // Creamos un nuevo PVector, que será el que devolveremos con el resultado del cálculo
    
    pvPos.x = m_pvIniPos.x; // La posición x de la partícula no cambia en la vibración.
    pvPos.y = m_pvIniPos.y + m_fAmpl*sin(-fT*2*PI/m_fFreq + m_fPhase); // La posición y varía según se describe en un MOHS
    
    return pvPos;
  }
}
